@@ -3,7 +3,7 @@
 /*
     Rubber Band Library
     An audio time-stretching and pitch-shifting library.
-    Copyright 2007-2022 Particular Programs Ltd.
+    Copyright 2007-2023 Particular Programs Ltd.
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License as
@@ -271,7 +271,8 @@ RubberBandR3PitchShifter::RubberBandR3PitchShifter(int sampleRate, size_t channe
     m_stretcher(new RubberBandStretcher
                 (sampleRate, channels,
                  RubberBandStretcher::OptionProcessRealTime |
-                 RubberBandStretcher::OptionEngineFiner)),
+                 RubberBandStretcher::OptionEngineFiner |
+                 RubberBandStretcher::OptionPitchHighConsistency)),
     m_sampleRate(sampleRate),
     m_channels(channels)
 {
