@@ -3,7 +3,7 @@
 /*
     Rubber Band Library
     An audio time-stretching and pitch-shifting library.
-    Copyright 2007-2022 Particular Programs Ltd.
+    Copyright 2007-2023 Particular Programs Ltd.
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License as
@@ -241,7 +241,7 @@ int main(int argc, char **argv)
         cerr << endl;
 	cerr << "Rubber Band" << endl;
         cerr << "An audio time-stretching and pitch-shifting library and utility program." << endl;
-	cerr << "Copyright 2007-2022 Particular Programs Ltd." << endl;
+	cerr << "Copyright 2007-2023 Particular Programs Ltd." << endl;
         cerr << endl;
 	cerr << "   Usage: " << myName << " [options] <infile.wav> <outfile.wav>" << endl;
         cerr << endl;
@@ -303,11 +303,23 @@ int main(int argc, char **argv)
         cerr << "  the pitch, retaining the original timbre of vocals and instruments in a" << endl;
         cerr << "  recognisable way." << endl;
         cerr << endl;
+        cerr << "         --centre-focus   Preserve focus of centre material in stereo" << endl;
+        cerr << endl;
+        cerr << "  This option assumes that any 2-channel audio files are stereo and treats" << endl;
+        cerr << "  them in a way that improves focus of the centre material at a small expense" << endl;
+        cerr << "  in quality of the individual channels. In v3.2+ (and R2) this also" << endl;
+        cerr << "  preserves mono compatibility, which the default options do not always." << endl;
+        cerr << endl;
         if (fullHelp || !isR3) {
             cerr << "  -c<N>, --crisp <N>      Crispness (N = 0,1,2,3,4,5,6); default 5" << endl;
             cerr << endl;
-            cerr << "  This option only has an effect when using the R2 (faster) engine. See below" << endl;
-            cerr << "  for details of the different levels." << endl;
+            cerr << "  This option only has an effect when using the R2 (faster) engine. See" << endl;
+            if (fullHelp) {
+                cerr << "  below ";
+            } else {
+                cerr << "  the full help ";
+            }
+            cerr << "for details of the different levels." << endl;
             cerr << endl;
         }
         if (fullHelp) {
@@ -332,8 +344,6 @@ int main(int argc, char **argv)
             cerr << "         --window-short   Use shorter processing window (with the R3 engine" << endl;
             cerr << "                          this is effectively a quick \"draft mode\")" << endl;
             cerr << "         --pitch-hq       In RT mode, use a slower, higher quality pitch shift" << endl;
-            cerr << "         --centre-focus   Preserve focus of centre material in stereo" << endl;
-            cerr << "                          (at a cost in width and individual channel quality)" << endl;
             cerr << "         --ignore-clipping Ignore clipping at output; the default is to restart" << endl;
             cerr << "                          with reduced gain if clipping occurs" << endl;
             cerr << "  -L,    --loose          [Accepted for compatibility but ignored; always off]" << endl;
@@ -727,10 +737,12 @@ int main(int argc, char **argv)
                 cerr << "Pass 1: Studying..." << endl;
             }
 
-            while (frame < sfinfo.frames) {
+            bool final = false;
+            
+            while (!final) {
 
                 int count = -1;
-                if ((count = sf_readf_float(sndfile, ibuf, bs)) <= 0) break;
+                if ((count = sf_readf_float(sndfile, ibuf, bs)) < 0) break;
         
                 for (size_t c = 0; c < channels; ++c) {
                     for (int i = 0; i < count; ++i) {
@@ -738,7 +750,10 @@ int main(int argc, char **argv)
                     }
                 }
 
-                bool final = (frame + bs >= sfinfo.frames);
+                final = (frame + bs >= sfinfo.frames);
+                if (count == 0) {
+                    final = true;
+                }
 
                 ts.study(cbuf, count, final);
 
@@ -798,8 +813,10 @@ int main(int argc, char **argv)
                 }
             }
         }                
+
+        bool final = false;
         
-        while (frame < sfinfo.frames) {
+        while (!final) {
 
             thisBlockSize = bs;
 
@@ -836,8 +853,15 @@ int main(int argc, char **argv)
                 }
             }
 
-            bool final = (frame + thisBlockSize >= sfinfo.frames);
+            final = (frame + thisBlockSize >= sfinfo.frames);
 
+            if (count == 0) {
+                if (debug > 1) {
+                    cerr << "at frame " << frame << " of " << sfinfo.frames << ", read count = " << count << ": marking final as true" << endl;
+                }
+                final = true;
+            }
+            
             if (debug > 2) {
                 cerr << "count = " << count << ", bs = " << thisBlockSize << ", frame = " << frame << ", frames = " << sfinfo.frames << ", final = " << final << endl;
             }
