@@ -3,7 +3,7 @@
 /*
     Rubber Band Library
     An audio time-stretching and pitch-shifting library.
-    Copyright 2007-2022 Particular Programs Ltd.
+    Copyright 2007-2023 Particular Programs Ltd.
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License as
@@ -77,7 +77,8 @@
 
 #define BQ_R__ R__
 
-using namespace std;
+using std::cerr;
+using std::endl;
 
 namespace RubberBand {
 
@@ -694,7 +695,7 @@ D_SRC::resampleInterleaved(float *const BQ_R__ out,
     // expected output, and (b) when the ratio has just changed, we
     // should aim to supply a shortish block next
     
-    if (outcount > int(ceil(incount * ratio) + 5)) {
+    if (!final && (outcount > int(ceil(incount * ratio) + 5))) {
         outcount = int(ceil(incount * ratio) + 5);
     }
 
@@ -753,6 +754,10 @@ D_SRC::resampleInterleaved(float *const BQ_R__ out,
 #endif
     }
 
+//    cerr << "Resampler::process: final = " << final << ", incount = "
+//         << incount << ", outcount = " << outcount << ", ratio = "
+//         << ratio << ", gen = " << data.output_frames_gen << endl;
+    
     return (int)data.output_frames_gen;
 }
 
@@ -1173,26 +1178,37 @@ D_Speex::doResample(const float *data_in, unsigned int &uincount,
     if (final) {
         int actual = int(uoutcount);
         int expected = std::min(initial_outcount, int(round(uincount * ratio)));
-        if (actual < expected) {
+        float *pad = nullptr;
+        while (actual < expected) {
             unsigned int final_out = expected - actual;
             unsigned int final_in = (unsigned int)(round(final_out / ratio));
-            if (final_in > 0) {
-                float *pad = allocate_and_zero<float>(final_in * m_channels);
+            if (final_in == 0) {
+                break;
+            } else {
+                if (!pad) {
+                    pad = allocate_and_zero<float>(final_in * m_channels);
+                }
                 err = speex_resampler_process_interleaved_float
                     (m_resampler,
                      pad, &final_in,
                      data_out + actual * m_channels, &final_out);
-                deallocate(pad);
+                actual += final_out;
                 uoutcount += final_out;
                 if (err) {
                     cerr << "Resampler::Resampler: Speex resampler returned error "
                          << err << endl;
+                    if (pad) {
+                        deallocate(pad);
+                    }
 #ifndef NO_EXCEPTIONS
                     throw Resampler::ImplementationError;
 #endif
                 }
             }
         }
+        if (pad) {
+            deallocate(pad);
+        }
     }
 }
 
