@@ -3,7 +3,7 @@
 /*
     Rubber Band Library
     An audio time-stretching and pitch-shifting library.
-    Copyright 2007-2022 Particular Programs Ltd.
+    Copyright 2007-2023 Particular Programs Ltd.
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License as
@@ -61,7 +61,13 @@ R3Stretcher::R3Stretcher(Parameters parameters,
     m_mode(ProcessMode::JustCreated)
 {
     Profiler profiler("R3Stretcher::R3Stretcher");
-    
+
+    initialise();
+}
+
+void
+R3Stretcher::initialise()
+{
     m_log.log(1, "R3Stretcher::R3Stretcher: rate, options",
               m_parameters.sampleRate, m_parameters.options);
     m_log.log(1, "R3Stretcher::R3Stretcher: initial time ratio and pitch scale",
@@ -99,14 +105,19 @@ R3Stretcher::R3Stretcher(Parameters parameters,
     int inRingBufferSize = getWindowSourceSize() * 16;
     int outRingBufferSize = getWindowSourceSize() * 16;
 
+    int hopBufferSize =
+        2 * std::max(m_limits.maxInhop, m_limits.maxPreferredOuthop);
+    
+    m_channelData.clear();
+    
     for (int c = 0; c < m_parameters.channels; ++c) {
         m_channelData.push_back(std::make_shared<ChannelData>
                                 (segmenterParameters,
                                  classifierParameters,
-                                 m_guideConfiguration.longestFftSize,
                                  getWindowSourceSize(),
                                  inRingBufferSize,
-                                 outRingBufferSize));
+                                 outRingBufferSize,
+                                 hopBufferSize));
         for (int b = 0; b < m_guideConfiguration.fftBandLimitCount; ++b) {
             const auto &band = m_guideConfiguration.fftBandLimits[b];
             int fftSize = band.fftSize;
@@ -115,6 +126,8 @@ R3Stretcher::R3Stretcher(Parameters parameters,
                 (fftSize, m_guideConfiguration.longestFftSize);
         }
     }
+
+    m_scaleData.clear();
     
     for (int b = 0; b < m_guideConfiguration.fftBandLimitCount; ++b) {
         const auto &band = m_guideConfiguration.fftBandLimits[b];
@@ -141,9 +154,6 @@ R3Stretcher::R3Stretcher(Parameters parameters,
 
     calculateHop();
 
-    m_prevInhop = m_inhop;
-    m_prevOuthop = int(round(m_inhop * getEffectiveRatio()));
-
     if (!m_inhop.is_lock_free()) {
         m_log.log(0, "R3Stretcher: WARNING: std::atomic<int> is not lock-free");
     }
@@ -213,7 +223,7 @@ R3Stretcher::setPitchScale(double scale)
     if (!isRealTime()) {
         if (m_mode == ProcessMode::Studying ||
             m_mode == ProcessMode::Processing) {
-            m_log.log(0, "R3Stretcher::setTimeRatio: Cannot set pitch scale while studying or processing in non-RT mode");
+            m_log.log(0, "R3Stretcher::setPitchScale: Cannot set pitch scale while studying or processing in non-RT mode");
             return;
         }
     }
@@ -229,7 +239,7 @@ R3Stretcher::setFormantScale(double scale)
     if (!isRealTime()) {
         if (m_mode == ProcessMode::Studying ||
             m_mode == ProcessMode::Processing) {
-            m_log.log(0, "R3Stretcher::setTimeRatio: Cannot set formant scale while studying or processing in non-RT mode");
+            m_log.log(0, "R3Stretcher::setFormantScale: Cannot set formant scale while studying or processing in non-RT mode");
             return;
         }
     }
@@ -382,12 +392,17 @@ R3Stretcher::calculateHop()
     m_log.log(1, "calculateHop: inhop and mean outhop", m_inhop, m_inhop * ratio);
 
     if (m_inhop < m_limits.maxInhopWithReadahead) {
-        m_log.log(1, "calculateHop: using readahead");
+        m_log.log(1, "calculateHop: using readahead; maxInhopWithReadahead", m_limits.maxInhopWithReadahead);
         m_useReadahead = true;
     } else {
-        m_log.log(1, "calculateHop: not using readahead, inhop too long for buffer in current configuration");
+        m_log.log(1, "calculateHop: not using readahead; maxInhopWithReadahead", m_limits.maxInhopWithReadahead);
         m_useReadahead = false;
     }
+
+    if (m_mode == ProcessMode::JustCreated) {
+        m_prevInhop = m_inhop;
+        m_prevOuthop = int(round(m_inhop * getEffectiveRatio()));
+    }
 }
 
 void
@@ -495,7 +510,14 @@ R3Stretcher::getPreferredStartPad() const
     if (!isRealTime()) {
         return 0;
     } else {
-        return getWindowSourceSize() / 2;
+        bool resamplingBefore = false;
+        areWeResampling(&resamplingBefore, nullptr);
+        size_t pad = getWindowSourceSize() / 2;
+        if (resamplingBefore) {
+            return size_t(ceil(pad * m_pitchScale));
+        } else {
+            return pad;
+        }
     }
 }
 
@@ -505,8 +527,14 @@ R3Stretcher::getStartDelay() const
     if (!isRealTime()) {
         return 0;
     } else {
-        double factor = 0.5 / m_pitchScale;
-        return size_t(ceil(getWindowSourceSize() * factor));
+        bool resamplingBefore = false;
+        areWeResampling(&resamplingBefore, nullptr);
+        size_t delay = getWindowSourceSize() / 2;
+        if (resamplingBefore) {
+            return delay;
+        } else {
+            return size_t(ceil(delay / m_pitchScale));
+        }
     }
 }
 
@@ -519,7 +547,23 @@ R3Stretcher::getChannelCount() const
 void
 R3Stretcher::reset()
 {
+    m_inhop = 1;
+    m_prevInhop = 1;
+    m_prevOuthop = 1;
+    m_unityCount = 0;
+    m_startSkip = 0;
+    m_studyInputDuration = 0;
+    m_suppliedInputDuration = 0;
+    m_totalTargetDuration = 0;
+    m_consumedInputDuration = 0;
+    m_lastKeyFrameSurpassed = 0;
+    m_totalOutputDuration = 0;
+    m_keyFrameMap.clear();
+
+    m_mode = ProcessMode::JustCreated;
+
     m_calculator->reset();
+    
     if (m_resampler) {
         m_resampler->reset();
     }
@@ -532,18 +576,7 @@ R3Stretcher::reset()
         cd->reset();
     }
 
-    m_prevInhop = m_inhop;
-    m_prevOuthop = int(round(m_inhop * getEffectiveRatio()));
-
-    m_studyInputDuration = 0;
-    m_suppliedInputDuration = 0;
-    m_totalTargetDuration = 0;
-    m_consumedInputDuration = 0;
-    m_lastKeyFrameSurpassed = 0;
-    m_totalOutputDuration = 0;
-    m_keyFrameMap.clear();
-
-    m_mode = ProcessMode::JustCreated;
+    calculateHop();
 }
 
 void
@@ -580,27 +613,91 @@ R3Stretcher::getSamplesRequired() const
 {
     if (available() != 0) return 0;
     int rs = m_channelData[0]->inbuf->getReadSpace();
+
+    m_log.log(2, "getSamplesRequired: read space and window source size", rs, getWindowSourceSize());
+
     if (rs < getWindowSourceSize()) {
-        return getWindowSourceSize() - rs;
+
+        int req = getWindowSourceSize() - rs;
+        
+        bool resamplingBefore = false;
+        areWeResampling(&resamplingBefore, nullptr);
+
+        if (!resamplingBefore) {
+            return req;
+        } else {
+            int adjusted = int(ceil(double(req) * m_pitchScale));
+            m_log.log(2, "getSamplesRequired: resamplingBefore is true, req and adjusted", req, adjusted);
+            return adjusted;
+        }
+            
     } else {
         return 0;
     }
 }
 
 void
-R3Stretcher::setMaxProcessSize(size_t n)
+R3Stretcher::setMaxProcessSize(size_t requested)
 {
-    size_t oldSize = m_channelData[0]->inbuf->getSize();
-    size_t newSize = getWindowSourceSize() + n;
-
-    if (newSize > oldSize) {
-        m_log.log(1, "setMaxProcessSize: resizing from and to", oldSize, newSize);
-        for (int c = 0; c < m_parameters.channels; ++c) {
-            m_channelData[c]->inbuf = std::unique_ptr<RingBuffer<float>>
-                (m_channelData[c]->inbuf->resized(newSize));
-        }
+    m_log.log(2, "R3Stretcher::setMaxProcessSize", requested);
+    
+    int n = m_limits.overallMaxProcessSize;
+    if (requested > size_t(n)) {
+        m_log.log(0, "R3Stretcher::setMaxProcessSize: request exceeds overall limit", requested, n);
     } else {
-        m_log.log(1, "setMaxProcessSize: nothing to be done, newSize <= oldSize", newSize, oldSize);
+        n = int(requested);
+    }
+
+    ensureInbuf(n * 2, false);
+    ensureOutbuf(n * 8, false);
+}
+
+size_t
+R3Stretcher::getProcessSizeLimit() const
+{
+    return m_limits.overallMaxProcessSize;
+}
+
+void
+R3Stretcher::ensureInbuf(int required, bool warn)
+{
+    int ws = m_channelData[0]->inbuf->getWriteSpace();
+    if (required < ws) {
+        return;
+    }
+    if (warn) {
+        m_log.log(0, "R3Stretcher::ensureInbuf: WARNING: Forced to increase input buffer size. Either setMaxProcessSize was not properly called, process is being called repeatedly without retrieve, or an internal error has led to an incorrect resampler output calculation. Samples to write and space available", required, ws);
+    }
+    size_t oldSize = m_channelData[0]->inbuf->getSize();
+    size_t newSize = oldSize - ws + required;
+    if (newSize < oldSize * 2) newSize = oldSize * 2;
+    m_log.log(warn ? 0 : 2, "R3Stretcher::ensureInbuf: old and new sizes", oldSize, newSize);
+    for (int c = 0; c < m_parameters.channels; ++c) {
+        auto newBuf = m_channelData[c]->inbuf->resized(newSize);
+        m_channelData[c]->inbuf = std::unique_ptr<RingBuffer<float>>(newBuf);
+        // mixdown is used for mid-side mixing as well as the single
+        // hop output mix, so it needs to be enough to match the inbuf
+        m_channelData[c]->mixdown.resize(newSize, 0.f);
+    }
+}
+
+void
+R3Stretcher::ensureOutbuf(int required, bool warn)
+{
+    int ws = m_channelData[0]->outbuf->getWriteSpace();
+    if (required < ws) {
+        return;
+    }
+    if (warn) {
+        m_log.log(0, "R3Stretcher::ensureOutbuf: WARNING: Forced to increase output buffer size. Using smaller process blocks or an artificially larger value for setMaxProcessSize may avoid this. Samples to write and space available", required, ws);
+    }
+    size_t oldSize = m_channelData[0]->outbuf->getSize();
+    size_t newSize = oldSize - ws + required;
+    if (newSize < oldSize * 2) newSize = oldSize * 2;
+    m_log.log(warn ? 0 : 2, "R3Stretcher::ensureOutbuf: old and new sizes", oldSize, newSize);
+    for (int c = 0; c < m_parameters.channels; ++c) {
+        auto newBuf = m_channelData[c]->outbuf->resized(newSize);
+        m_channelData[c]->outbuf = std::unique_ptr<RingBuffer<float>>(newBuf);
     }
 }
 
@@ -614,6 +711,13 @@ R3Stretcher::process(const float *const *input, size_t samples, bool final)
         return;
     }
 
+    int n = m_limits.overallMaxProcessSize;
+    if (samples > size_t(n)) {
+        m_log.log(0, "R3Stretcher::process: request exceeds overall limit", samples, n);
+    } else {
+        n = int(samples);
+    }
+
     if (!isRealTime()) {
 
         if (m_mode == ProcessMode::Studying) {
@@ -652,8 +756,12 @@ R3Stretcher::process(const float *const *input, size_t samples, bool final)
             // when the ratio changes.
             int pad = getWindowSourceSize() / 2;
             m_log.log(1, "offline mode: prefilling with", pad);
+            ensureInbuf(pad);
             for (int c = 0; c < m_parameters.channels; ++c) {
-                m_channelData[c]->inbuf->zero(pad);
+                int zeroed = m_channelData[c]->inbuf->zero(pad);
+                if (zeroed != pad) {
+                    m_log.log(0, "R3Stretcher: WARNING: too few padding samples written", zeroed, pad);
+                }
             }
 
             // NB by the time we skip this later we may have resampled
@@ -663,45 +771,28 @@ R3Stretcher::process(const float *const *input, size_t samples, bool final)
         }
     }
 
-    if (final) {
-        // We don't distinguish between Finished and "draining, but
-        // haven't yet delivered all the samples" because the
-        // distinction is meaningless internally - it only affects
-        // whether available() finds any samples in the buffer
-        m_mode = ProcessMode::Finished;
-    } else {
-        m_mode = ProcessMode::Processing;
-    }
-
     bool resamplingBefore = false;
     areWeResampling(&resamplingBefore, nullptr);
 
     int channels = m_parameters.channels;
     int inputIx = 0;
 
-    while (inputIx < int(samples)) {
+    if (n == 0 && final) {
 
-        int remaining = int(samples) - inputIx;
-        int ws = m_channelData[0]->inbuf->getWriteSpace();
-
-        if (ws == 0) {
-            consume();
-            ws = m_channelData[0]->inbuf->getWriteSpace();
-        }
+        m_log.log(2, "process: no samples but final specified, consuming");
         
+        consume(true);
+
+    } else while (inputIx < n) {
+
+        int remaining = n - inputIx;
+
+        int ws = m_channelData[0]->inbuf->getWriteSpace();
         if (ws == 0) {
-            m_log.log(0, "R3Stretcher::process: WARNING: Forced to increase input buffer size. Either setMaxProcessSize was not properly called, process is being called repeatedly without retrieve, or an internal error has led to an incorrect resampler output calculation. Samples to write", remaining);
-            size_t oldSize = m_channelData[0]->inbuf->getSize();
-            size_t newSize = oldSize + remaining;
-            if (newSize < oldSize * 2) newSize = oldSize * 2;
-            m_log.log(0, "R3Stretcher::process: old and new sizes", oldSize, newSize);
-            for (int c = 0; c < m_parameters.channels; ++c) {
-                auto newBuf = m_channelData[c]->inbuf->resized(newSize);
-                m_channelData[c]->inbuf =
-                    std::unique_ptr<RingBuffer<float>>(newBuf);
-            }
-            continue;
+            consume(false);
         }
+        ensureInbuf(remaining);
+        ws = m_channelData[0]->inbuf->getWriteSpace();
         
         if (resamplingBefore) {
 
@@ -716,32 +807,66 @@ R3Stretcher::process(const float *const *input, size_t samples, bool final)
             int maxResampleInput = int(floor(maxResampleOutput * m_pitchScale));
             int resampleInput = std::min(remaining, maxResampleInput);
             if (resampleInput == 0) resampleInput = 1;
+
+            m_log.log(2, "R3Stretcher::process: resamplingBefore is true, resampleInput and maxResampleOutput", resampleInput, maxResampleOutput);
+            
+            prepareInput(input, inputIx, resampleInput);
+
+            bool finalHop = (final && inputIx + resampleInput >= n);
             
             int resampleOutput = m_resampler->resample
                 (m_channelAssembly.resampled.data(),
                  maxResampleOutput,
-                 input,
+                 m_channelAssembly.input.data(),
                  resampleInput,
                  1.0 / m_pitchScale,
-                 final);
+                 finalHop);
 
             inputIx += resampleInput;
-
+            
             for (int c = 0; c < m_parameters.channels; ++c) {
-                m_channelData[c]->inbuf->write
+
+                int written = m_channelData[c]->inbuf->write
                     (m_channelData.at(c)->resampled.data(),
                      resampleOutput);
+
+                if (written != resampleOutput) {
+                    m_log.log(0, "R3Stretcher: WARNING: too few samples written to input buffer from resampler", written, resampleOutput);
+                }
             }
 
         } else {
             int toWrite = std::min(ws, remaining);
+
+            m_log.log(2, "process: resamplingBefore is false, writing to inbuf from supplied data, former read space and samples being added", m_channelData[0]->inbuf->getReadSpace(), toWrite);
+
+            prepareInput(input, inputIx, toWrite);
+            
             for (int c = 0; c < m_parameters.channels; ++c) {
-                m_channelData[c]->inbuf->write (input[c] + inputIx, toWrite);
+
+                int written = m_channelData[c]->inbuf->write
+                    (m_channelAssembly.input[c], toWrite);
+
+                if (written != toWrite) {
+                    m_log.log(0, "R3Stretcher: WARNING: too few samples written to input buffer", written, toWrite);
+                }
             }
+            
             inputIx += toWrite;
         }
         
-        consume();
+        consume(final && inputIx >= n);
+    }
+
+    if (final) {
+        // We don't distinguish between Finished and "draining, but
+        // haven't yet delivered all the samples" because the
+        // distinction is meaningless internally - it only affects
+        // whether available() finds any samples in the buffer
+        m_log.log(1, "final is set, entering Finished mode");
+        m_mode = ProcessMode::Finished;
+    } else {
+        m_mode = ProcessMode::Processing;
     }
 }
 
@@ -763,6 +888,8 @@ R3Stretcher::retrieve(float *const *output, size_t samples) const
     
     int got = samples;
     
+    m_log.log(2, "retrieve: requested, outbuf has", samples, m_channelData[0]->outbuf->getReadSpace());
+    
     for (int c = 0; c < m_parameters.channels; ++c) {
         int gotHere = m_channelData[c]->outbuf->read(output[c], got);
         if (gotHere < got) {
@@ -773,11 +900,52 @@ R3Stretcher::retrieve(float *const *output, size_t samples) const
         }
     }
 
+    if (useMidSide()) {
+        for (int i = 0; i < got; ++i) {
+            float m = output[0][i];
+            float s = output[1][i];
+            float l = m + s;
+            float r = m - s;
+            output[0][i] = l;
+            output[1][i] = r;
+        }
+    }
+    
+    m_log.log(2, "retrieve: returning, outbuf now has", got, m_channelData[0]->outbuf->getReadSpace());
+
     return got;
 }
 
 void
-R3Stretcher::consume()
+R3Stretcher::prepareInput(const float *const *input, int ix, int n)
+{
+    if (useMidSide()) {
+        auto &c0 = m_channelData.at(0)->mixdown;
+        auto &c1 = m_channelData.at(1)->mixdown;
+        int bufsize = c0.size();
+        if (n > bufsize) {
+            m_log.log(0, "R3Stretcher::prepareInput: WARNING: called with size greater than mixdown buffer length", n, bufsize);
+            n = bufsize;
+        }
+        for (int i = 0; i < n; ++i) {
+            float l = input[0][i + ix];
+            float r = input[1][i + ix];
+            float m = (l + r) / 2.f;
+            float s = (l - r) / 2.f;
+            c0[i] = m;
+            c1[i] = s;
+        }
+        m_channelAssembly.input[0] = m_channelData.at(0)->mixdown.data();
+        m_channelAssembly.input[1] = m_channelData.at(1)->mixdown.data();
+    } else {
+        for (int c = 0; c < m_parameters.channels; ++c) {
+            m_channelAssembly.input[c] = input[c] + ix;
+        }
+    }
+}
+
+void
+R3Stretcher::consume(bool final)
 {
     Profiler profiler("R3Stretcher::consume");
     
@@ -828,20 +996,23 @@ R3Stretcher::consume()
     }
 
     auto &cd0 = m_channelData.at(0);
-    
-    while (cd0->outbuf->getWriteSpace() >= outhop) {
+
+    m_log.log(2, "consume: write space and outhop", cd0->outbuf->getWriteSpace(), outhop);
+        
+    // NB our ChannelData, ScaleData, and ChannelScaleData maps
+    // contain shared_ptrs; whenever we retain one of them in a
+    // variable, we do so by reference to avoid copying the shared_ptr
+    // (as that is not realtime safe). Same goes for the map iterators
+
+    while (true) {
 
         Profiler profiler("R3Stretcher::consume/loop");
-        
-        // NB our ChannelData, ScaleData, and ChannelScaleData maps
-        // contain shared_ptrs; whenever we retain one of them in a
-        // variable, we do so by reference to avoid copying the
-        // shared_ptr (as that is not realtime safe). Same goes for
-        // the map iterators
 
         int readSpace = cd0->inbuf->getReadSpace();
+        m_log.log(2, "consume: read space", readSpace);
+
         if (readSpace < getWindowSourceSize()) {
-            if (m_mode == ProcessMode::Finished) {
+            if (final) {
                 if (readSpace == 0) {
                     int fill = cd0->scales.at(longest)->accumulatorFill;
                     if (fill == 0) {
@@ -856,6 +1027,8 @@ R3Stretcher::consume()
             }
         }
 
+        ensureOutbuf(outhop);
+        
         // Analysis
         
         for (int c = 0; c < channels; ++c) {
@@ -882,6 +1055,7 @@ R3Stretcher::consume()
                  m_channelAssembly.prevMag.data(),
                  m_guideConfiguration,
                  m_channelAssembly.guidance.data(),
+                 useMidSide(),
                  m_prevInhop,
                  m_prevOuthop);
         }
@@ -905,13 +1079,18 @@ R3Stretcher::consume()
                 m_channelAssembly.mixdown[c] = cd->mixdown.data();
                 m_channelAssembly.resampled[c] = cd->resampled.data();
             }
+
+            bool finalHop = (final &&
+                             readSpace < inhop &&
+                             cd0->scales.at(longest)->accumulatorFill <= outhop);
+            
             resampledCount = m_resampler->resample
                 (m_channelAssembly.resampled.data(),
                  m_channelData[0]->resampled.size(),
                  m_channelAssembly.mixdown.data(),
                  outhop,
                  1.0 / m_pitchScale,
-                 m_mode == ProcessMode::Finished && readSpace < inhop);
+                 finalHop);
         }
 
         // Emit
@@ -933,8 +1112,8 @@ R3Stretcher::consume()
 
         int advanceCount = inhop;
         if (advanceCount > readSpace) {
-            // This should happen only when draining (Finished)
-            if (m_mode != ProcessMode::Finished) {
+            // This should happen only when draining
+            if (!final) {
                 m_log.log(0, "R3Stretcher: WARNING: readSpace < inhop when processing is not yet finished", readSpace, inhop);
             }
             advanceCount = readSpace;
@@ -942,12 +1121,19 @@ R3Stretcher::consume()
         
         for (int c = 0; c < channels; ++c) {
             auto &cd = m_channelData.at(c);
+            int written = 0;
             if (resamplingAfter) {
-                cd->outbuf->write(cd->resampled.data(), writeCount);
+                written = cd->outbuf->write(cd->resampled.data(), writeCount);
             } else {
-                cd->outbuf->write(cd->mixdown.data(), writeCount);
+                written = cd->outbuf->write(cd->mixdown.data(), writeCount);
+            }
+            if (written != writeCount) {
+                m_log.log(0, "R3Stretcher: WARNING: too few samples written to output buffer", written, writeCount);
+            }
+            int skipped = cd->inbuf->skip(advanceCount);
+            if (skipped != advanceCount) {
+                m_log.log(0, "R3Stretcher: WARNING: too few samples advanced", skipped, advanceCount);
             }
-            cd->inbuf->skip(advanceCount);
         }
 
         m_consumedInputDuration += advanceCount;
@@ -957,7 +1143,10 @@ R3Stretcher::consume()
             int rs = cd0->outbuf->getReadSpace();
             int toSkip = std::min(m_startSkip, rs);
             for (int c = 0; c < channels; ++c) {
-                m_channelData.at(c)->outbuf->skip(toSkip);
+                int skipped = m_channelData.at(c)->outbuf->skip(toSkip);
+                if (skipped != toSkip) {
+                    m_log.log(0, "R3Stretcher: WARNING: too few samples skipped at output", skipped, toSkip);
+                }
             }
             m_startSkip -= toSkip;
             m_totalOutputDuration = rs - toSkip;
@@ -966,6 +1155,8 @@ R3Stretcher::consume()
         m_prevInhop = inhop;
         m_prevOuthop = outhop;
     }
+
+    m_log.log(2, "consume: write space reduced to", cd0->outbuf->getWriteSpace());
 }
 
 void
@@ -1200,6 +1391,15 @@ R3Stretcher::analyseChannel(int c, int inhop, int prevInhop, int prevOuthop)
         m_parameters.options & RubberBandStretcher::OptionChannelsTogether;
 
     double magMean = v_mean(classifyScale->mag.data() + 1, classify/2);
+
+    bool resetOnSilence = true;
+    if (useMidSide() && c == 1) {
+        // Do not phase reset on silence in the side channel - the
+        // reset is propagated across to the mid channel, giving
+        // constant resets for e.g. mono material in a stereo
+        // configuration
+        resetOnSilence = false;
+    }
     
     if (m_useReadahead) {
         m_guide.updateGuidance(ratio,
@@ -1214,6 +1414,7 @@ R3Stretcher::analyseChannel(int c, int inhop, int prevInhop, int prevOuthop)
                                m_unityCount,
                                isRealTime(),
                                tighterChannelLock,
+                               resetOnSilence,
                                cd->guidance);
     } else {
         m_guide.updateGuidance(ratio,
@@ -1228,6 +1429,7 @@ R3Stretcher::analyseChannel(int c, int inhop, int prevInhop, int prevOuthop)
                                m_unityCount,
                                isRealTime(),
                                tighterChannelLock,
+                               resetOnSilence,
                                cd->guidance);
     }
     
