@@ -3,7 +3,7 @@
 /*
     Rubber Band Library
     An audio time-stretching and pitch-shifting library.
-    Copyright 2007-2022 Particular Programs Ltd.
+    Copyright 2007-2023 Particular Programs Ltd.
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License as
@@ -42,6 +42,7 @@
 
 #include <map>
 #include <memory>
+#include <atomic>
 
 namespace RubberBand
 {
@@ -92,6 +93,7 @@ public:
 
     void setExpectedInputDuration(size_t samples);
     void setMaxProcessSize(size_t samples);
+    size_t getProcessSizeLimit() const;
     
     void setDebugLevel(int level) {
         m_log.setDebugLevel(level);
@@ -109,13 +111,15 @@ protected:
         int minInhop;
         int maxInhopWithReadahead;
         int maxInhop;
+        int overallMaxProcessSize;
         Limits(RubberBandStretcher::Options options, double rate) :
             // commented values are results when rate = 44100 or 48000
             minPreferredOuthop(roundUpDiv(rate, 512)), // 128
             maxPreferredOuthop(roundUpDiv(rate, 128)), // 512
             minInhop(1),
-            maxInhopWithReadahead(roundUpDiv(rate, 32)), // 1024
-            maxInhop(roundUpDiv(rate, 32)) {             // 1024
+            maxInhopWithReadahead(roundUpDiv(rate, 64)), // 1024
+            maxInhop(roundUpDiv(rate, 32)),              // 2048
+            overallMaxProcessSize(524288) {
             if (options & RubberBandStretcher::OptionWindowShort) {
                 // See note in calculateHop
                 minPreferredOuthop = roundUpDiv(rate, 256); // 256
@@ -172,6 +176,7 @@ protected:
 
         void reset() {
             v_zero(prevMag.data(), prevMag.size());
+            v_zero(pendingKick.data(), pendingKick.size());
             v_zero(accumulator.data(), accumulator.size());
             accumulatorFill = 0;
         }
@@ -226,10 +231,10 @@ protected:
         std::unique_ptr<FormantData> formant;
         ChannelData(BinSegmenter::Parameters segmenterParameters,
                     BinClassifier::Parameters classifierParameters,
-                    int longestFftSize,
                     int windowSourceSize,
                     int inRingBufferSize,
-                    int outRingBufferSize) :
+                    int outRingBufferSize,
+                    int hopBufferSize) :
             scales(),
             windowSource(windowSourceSize, 0.0),
             readahead(segmenterParameters.fftSize),
@@ -241,8 +246,8 @@ protected:
                                BinClassifier::Classification::Residual),
             segmenter(new BinSegmenter(segmenterParameters)),
             segmentation(), prevSegmentation(), nextSegmentation(),
-            mixdown(longestFftSize, 0.f),
-            resampled(outRingBufferSize, 0.f),
+            mixdown(inRingBufferSize, 0.f),
+            resampled(hopBufferSize, 0.f),
             inbuf(new RingBuffer<float>(inRingBufferSize)),
             outbuf(new RingBuffer<float>(outRingBufferSize)),
             formant(new FormantData(segmenterParameters.fftSize)) { }
@@ -252,6 +257,9 @@ protected:
             segmentation = BinSegmenter::Segmentation();
             prevSegmentation = BinSegmenter::Segmentation();
             nextSegmentation = BinSegmenter::Segmentation();
+            for (size_t i = 0; i < nextClassification.size(); ++i) {
+                nextClassification[i] = BinClassifier::Classification::Residual;
+            }
             inbuf->reset();
             outbuf->reset();
             for (auto &s : scales) {
@@ -263,6 +271,7 @@ protected:
     struct ChannelAssembly {
         // Vectors of bare pointers, used to package container data
         // from different channels into arguments for PhaseAdvance
+        FixedVector<const float *> input;
         FixedVector<process_t *> mag;
         FixedVector<process_t *> phase;
         FixedVector<process_t *> prevMag;
@@ -271,6 +280,7 @@ protected:
         FixedVector<float *> mixdown;
         FixedVector<float *> resampled;
         ChannelAssembly(int channels) :
+            input(channels, nullptr),
             mag(channels, nullptr), phase(channels, nullptr),
             prevMag(channels, nullptr), guidance(channels, nullptr),
             outPhase(channels, nullptr), mixdown(channels, nullptr),
@@ -350,8 +360,12 @@ protected:
     };
     ProcessMode m_mode;
 
-    void consume();
+    void initialise();
+    void prepareInput(const float *const *input, int ix, int n);
+    void consume(bool final);
     void createResampler();
+    void ensureInbuf(int, bool warn = true);
+    void ensureOutbuf(int, bool warn = true);
     void calculateHop();
     void updateRatioFromMap();
     void analyseChannel(int channel, int inhop, int prevInhop, int prevOuthop);
@@ -469,6 +483,12 @@ protected:
         }
     }        
 
+    bool useMidSide() const {
+        return m_parameters.channels == 2 &&
+            (m_parameters.options &
+             RubberBandStretcher::OptionChannelsTogether);
+    }
+    
     bool isSingleWindowed() const {
         return m_parameters.options &
             RubberBandStretcher::OptionWindowShort;
